@@ -3,7 +3,7 @@ package Apache::FileManager;
 
 =head1 NAME
 
-Apache::FileManager - Apache file manager
+Apache::FileManager - apache mod_perl file manager
 
 =head1 SYNOPSIS
 
@@ -20,9 +20,9 @@ Or call from your own mod_perl script
 
 =head1 DESCRIPTION
 
-The Apache::FileManager module is a simple HTML file manager. It provides file manipulations such as cut, copy, paste, delete, rename, extract archive, create directory, and upload files. All of these can be enacted on one or more files at a time (except rename). This module requires the client to have Java-script, and cookies enabled.
+The Apache::FileManager module is a simple HTML file manager. It provides file manipulations such as cut, copy, paste, delete, rename, extract archive, create directory, and upload files. The interface is clean and simple, and configuration is a breeze.
 
-The Apache::FileManager also can be used for a development site. The document tree can then be copied to the production server with the click of a button in the File Manager via an rsync. If this functionality is wanted, you must also install File::Rsync.
+For those of you who are up to the challenge, you can configure Apache::FileManager on run on a development server and update your live server with the click on a button. 
 
 =head1 SPECIAL NOTES
 
@@ -33,12 +33,10 @@ run the following commands before you begin.
 chown -R nobody /web/xyz/htdocs
 chmod -R 755 /web/xyz/htdocs
 
-The extract functionality only works with tarballs and zips. Is there demand for anything else?
+The extract functionality only works with tarballs and zips.
 
 =head1 RSYNC FEATURE
 
-Warning! rsync will delete files on the production server that do not exist on the development server for the directory specified on the production server specified by the RSYNC_TO directive.
-
 To use the rync functionality you must have ssh, rsync, and the File::Rsync perl module installed on the development server. You also must have an sshd running on the production server.
 
 Make sure you always fully qualify your server names so you don't have different values in your known hosts file.
@@ -92,7 +90,7 @@ You also need to specify the production server in the development server's web c
      <Location /FileManager>
        SetHandler           perl-script
        PerlHandler          Apache::FileManager
-       PerlSetVar           RSYNC_TO   production_server:/web/xyz/htdocs
+       PerlSetVar           RSYNC_TO   production_server:/web/xyz
      </Location>
 
 If your ssh path is not /usr/bin/ssh or /usr/local/bin/ssh, you also need to specify the path in the conf file or in the contructor with the directive SSH_PATH.
@@ -109,12 +107,12 @@ I am sure there are some.
 
 =head1 TODO
 
-It would be nice if you could choose a different base directory other then the document root in the constructor. I may do this sometime if I have a need to. If you want to contribute, send me your updates.
+It would be nice if you could choose a different base directory other then the document root in the constructor. I may do this sometime if I have a need to. If you want to contribute, let me know.
 
 =head1 AUTHOR
 
 Apache::FileManager was written by Philip Collins 
-E<lt>collins_p@yahoo.comE<gt>.
+E<lt>pmc@cpan.orgE<gt>.
 
 =cut
 
@@ -127,14 +125,12 @@ use File::Remove qw(remove);
 use File::stat;
 use Archive::Any;
 use POSIX qw(strftime);
-use Storable qw(freeze thaw);
-use MIME::Base64 qw(encode_base64 decode_base64);
 use CGI::Cookie;
 #use Data::Dumper;
 
 require 5.005_62;
 
-our $VERSION = '0.03';
+our $VERSION = '0.05';
 
 sub r      { return Apache::Request->instance( Apache->request ); }
 
@@ -155,6 +151,9 @@ sub new {
 
   my $o = bless $ref, $pack;
 
+  $$o{MESSAGE} = "";
+  $$o{JS} = "";
+
   # Is this filemanager rsync capable?
   $$o{'RSYNC_TO'} ||= r->dir_config('RSYNC_TO');
 
@@ -199,11 +198,12 @@ sub new {
   #document root
   my $dr = r->document_root;
 
+
   #verify current working directory
   $_ = r->param('FILEMANAGER_curr_dir');
-  s/\.//g; s/^\///; s/\/$//;
+  s/\.\./\./g; s/^\///; s/\/$//;
   my $curr_dir = $_;
-  
+
   #set current directory
   if (! chdir "$dr/$curr_dir") {
     chdir $dr;
@@ -216,7 +216,8 @@ sub new {
   if (r->param('FILEMANAGER_action') eq "paste") {
     if ($$o{buffer_type} eq "copy") {
       my @files = map { &fn_esc($dr."/".$_) } @{ $$o{buffer_filenames} };
-      copy \1, @files, ".";
+      my $count = copy \1, @files, ".";
+      $$o{MESSAGE} = "$count file(s) pasted.";
     } elsif ($$o{buffer_type} eq "cut") {
       for (@{ $$o{buffer_filenames} }) {
         my $file = $dr."/".$_;
@@ -226,17 +227,21 @@ sub new {
           if ($count) {
             remove \1, $file;
           }
+          $$o{MESSAGE} = "$count file(s) pasted.";
         } elsif (-f $file) {
-          move($file, ".");
+          my $count = move($file, ".");
+          $$o{MESSAGE} = "$count file(s) pasted.";
         }
       }
     }
+    $$o{JS} = "window.setcookie(cookie_name,'',-1);";
   }
 
   #delete selected files
   elsif (r->param('FILEMANAGER_action') eq "delete") {
     my @files = map { &fn_esc($dr."/".$_) } @sel_files;
-    remove \1, @files;
+    my $count = remove \1, @files;
+    $$o{MESSAGE} = "$count file(s) deleted.";
   }
 
   #extract zip and tar balls into current directory
@@ -245,19 +250,23 @@ sub new {
       my $esc = &fn_esc($dr."/".$f);
       my $archive = Archive::Any->new($esc);
       $archive->extract if defined $archive;
+      $$o{MESSAGE} = "Files extracted.";
     }
   }
 
   #upload files into current directory
   elsif (r->param('FILEMANAGER_action') eq "upload") {
+    my $count = 0;
     foreach my $i (1 .. 10) {
 
       my @ar = split /\/|\\/, r->param("FILEMANAGER_file$i");
       next if ($#ar == -1);
       my $filename = pop @ar;
-      $filename =~ s/[^\w\ \d\.]//g;
+      $filename =~ s/[^\w\ \d\.\-]//g;
       next if ($filename eq "");
 
+      $count++;
+
       my $up = r->upload("FILEMANAGER_file$i"); next if ! defined $up;
       my $in_fh = $up->fh; next if ! defined $in_fh;
 
@@ -270,15 +279,21 @@ sub new {
         print $out_fh $_;
       }
     }
+    $$o{MESSAGE} = "$count file(s) uploaded.";
   }
 
   #rename the first selected file
   elsif ( (r->param('FILEMANAGER_action') eq "rename") && ($#sel_files > -1) ) {
     my $file = $dr."/".$sel_files[0];
-    move($file, r->param('FILEMANAGER_rename'));
+    my $bool = move($file, r->param('FILEMANAGER_rename'));
+    if ($bool) {
+      $$o{MESSAGE} = "File renamed.";
+    } else {
+      $$o{MESSAGE} = "File could not be renamed.";
+    }
   }
 
-  #this is some future to do stuff (rsync htdocs to a production server)
+  #rsync (go live!) feature
   elsif (r->param('FILEMANAGER_action') eq "rsync") {
     $$o{'SSH_PATH'} ||= r->dir_config('SSH_PATH');
 
@@ -311,16 +326,15 @@ sub new {
 
   #create new directory in the current directory
   elsif (r->param('FILEMANAGER_action') eq "mkdir") {
-    mkdir r->param('FILEMANAGER_new_dir');
+    my $bool = mkdir r->param('FILEMANAGER_new_dir');
+    if ($bool) {
+      $$o{MESSAGE} = "New directory added.";
+    } else {
+      $$o{MESSAGE} = "Could not make directory.";
+    }
   }
 
   
-  #send copy and cut file arrays in a cookie
-#  my $encoded = &stringify([$cut_files, $copy_files]);
-#  my $server_name = r->server->server_hostname;
-#  my $cookie = new CGI::Cookie(-name=>"$server_name FileManager", -value=>$encoded);
-#  r->headers_out->add('Set-Cookie' => $cookie);
-
   return $o;
 }
 
@@ -345,15 +359,45 @@ sub print {
 
   my $rsync = "";
   if (exists $$o{'RSYNC_TO'}) {
-    $rsync = "<TD><A HREF=# style='text-decoration:none' onclick=\"var w=window.open('','RSYNC','scrollbars=yes,resizables=yes,width=400,height=500'); w.focus(); var d=w.document.open(); d.write('<HTML><BODY><BR><BR><BR><CENTER>Please wait synchronizing production server.<BR>This could take several minutes.</CENTER></BODY></HTML>'); d.close(); w.location.replace('".r->uri."?FILEMANAGER_action=rsync','RSYNC','scrollbars=yes,resizables=yes,width=400,height=500'); return false;\"><FONT COLOR=WHITE><B>go live!</B></FONT></A></TD>";
+    $rsync = "<TD><A HREF=# style='text-decoration:none' onclick=\"if (window.confirm('Are you sure you want to synchronize with the production server?')) {var w=window.open('','RSYNC','scrollbars=yes,resizables=yes,width=400,height=500'); w.focus(); var d=w.document.open(); d.write('<HTML><BODY><BR><BR><BR><CENTER>Please wait synchronizing production server.<BR>This could take several minutes.</CENTER></BODY></HTML>'); d.close(); w.location.replace('".r->uri."?FILEMANAGER_action=rsync','RSYNC','scrollbars=yes,resizables=yes,width=400,height=500'); } return false;\"><FONT COLOR=WHITE><B>go live!</B></FONT></A></TD>";
   }
 
   my $cookie_name = uc(r->server->server_hostname);
   $cookie_name =~ s/[^A-Z]//g;
   $cookie_name .= "_FM";
 
+  my $message = "<I><FONT COLOR=#990000>".$$o{MESSAGE}."</FONT></I>";
+
   print "
+$message
 <SCRIPT>
+  var cookie_name = '$cookie_name';
+
+  function getexpirydate(nodays){
+    var UTCstring;
+    Today = new Date();
+    nomilli=Date.parse(Today);
+    Today.setTime(nomilli+nodays*24*60*60*1000);
+    UTCstring = Today.toUTCString();
+    return UTCstring;
+  }
+
+  function getcookie(cookiename) {
+    var cookiestring=''+document.cookie;
+    var index1=cookiestring.indexOf(cookiename);
+    if (index1==-1 || cookiename=='') return ''; 
+    var index2=cookiestring.indexOf(';',index1);
+    if (index2==-1) index2=cookiestring.length; 
+    return unescape(cookiestring.substring(index1+cookiename.length+1,index2));
+  }
+  
+  function setcookie(name,value,duration){
+    cookiestring=name+'='+escape(value)+';EXPIRES='+getexpirydate(duration);
+    document.cookie=cookiestring;
+    if(!getcookie(name)){ return false; }
+    else{ return true; }
+  }
+
   function print_upload () {
     var w = window.open('','FileManagerUpload','scrollbars=yes,resizable=yes,width=360,height=440');
     var d = w.document.open();
@@ -366,15 +410,42 @@ sub print {
     w.focus();
   }
 
-  var cookie_name = '$cookie_name';
+  // make input check box form elements into an array ALL the time
+  function get_ckbox_array() {
+    var ar;
 
-  function setCookie(value) {
-    if (value != null && value != '') 
-      document.cookie = cookie_name + '=' + escape(value) + ';'
+    // no files
+    if (window.document.FileManager.FILEMANAGER_sel_files == null) {
+      ar = new Array();
+    }
+
+    // 1 file (no length)
+    else if (window.document.FileManager.FILEMANAGER_sel_files.length == null){
+      ar = [ window.document.FileManager.FILEMANAGER_sel_files ]; 
+    }
+
+    // more than one file
+    else {
+      ar = window.document.FileManager.FILEMANAGER_sel_files;
+    }
+    return ar;
   }
 
+  // get the number checked
+  function get_num_checked() {
+    var count = 0;
+    var ar = get_ckbox_array();
+    for (var i=0; i < ar.length; i++) {
+      if (ar[i].checked == true) {
+        count++;
+      }
+    }
+    return count;
+  }
+
+  // make cookie for checked filenames
   function save_names (type) {
-    var cb = window.document.FileManager.FILEMANAGER_sel_files;
+    var cb = get_ckbox_array();
     var ac = '';
     for (var i=0; i < cb.length; i++) {
       if (cb[i].checked == true) {
@@ -382,11 +453,29 @@ sub print {
         cb[i].checked = false;
       }
     }
-    ac = ac + type;
-    window.setCookie(ac);
+    if (ac == '') {
+      window.alert('Please select file(s) by clicking on the check boxes with the mouse.');
+    } else {
+      ac = ac + type;
+      window.setcookie(cookie_name,ac,1);
+    }
   }
+
+  //test if browser cookies are enabled
+  if (! window.document.cookie ) {
+    window.setcookie(cookie_name,'test',1);
+    if (! window.document.cookie) document.write('<H1><FONT COLOR=#990000>please enable cookies</FONT></H1>');
+    window.setcookie(cookie_name,'',-1);
+  } 
+
+$$o{JS}
+
 </SCRIPT>
 
+<NOSCRIPT>
+  <H1><FONT COLOR=#990000>please enable javascript</FONT></H1>
+</NOSCRIPT>
+
 <FORM NAME=FileManager ACTION='".r->uri."' METHOD=POST>
 <INPUT TYPE=HIDDEN NAME=FILEMANAGER_curr_dir VALUE='".r->param('FILEMANAGER_curr_dir')."'>
 <INPUT TYPE=HIDDEN NAME=FILEMANAGER_action VALUE=''>
@@ -405,16 +494,21 @@ sub print {
   <TD><A HREF=# onclick=\"var f=window.document.FileManager; f.submit();\" style='text-decoration:none'><FONT COLOR=WHITE><B>refresh</B></FONT></A></TD>
 
   <TD><A HREF=# onclick=\"window.save_names('cut'); return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>cut</B></FONT></A></TD>
+
   <TD><A HREF=# onclick=\"window.save_names('copy'); return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>copy</B></FONT></A></TD>
-  <TD><A HREF=# onclick=\"var f=window.document.FileManager; f.FILEMANAGER_action.value='paste'; f.submit(); return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>paste</B></FONT></A></TD>
-  <TD><A HREF=# onclick=\"var f=window.document.FileManager; f.FILEMANAGER_action.value='delete'; f.submit(); return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>delete</B></FONT></A></TD>
-  <TD><A HREF=# onclick=\"var f=window.document.FileManager; f.FILEMANAGER_action.value='rename'; var rv=window.prompt('enter new name',''); if ((rv != null)&&(rv != '')) { f.FILEMANAGER_rename.value=rv; f.submit(); } return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>rename</B></FONT></A></TD>
-  <TD><A HREF=# onclick=\"var f=window.document.FileManager; f.FILEMANAGER_action.value='extract'; f.submit(); return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>extract</B></FONT></A></TD>
+
+  <TD><A HREF=# onclick=\"if (window.getcookie(cookie_name) != '') { var f=window.document.FileManager; f.FILEMANAGER_action.value='paste'; f.submit(); } else { window.alert('Please select file(s) to paste by checking the file(s) first and clicking copy or cut.'); } return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>paste</B></FONT></A></TD>
+
+  <TD><A HREF=# onclick=\"var f=window.document.FileManager; if (get_num_checked() == 0) { window.alert('Please select a file to delete by clicking on a check box with the mouse.'); } else { f.FILEMANAGER_action.value='delete'; f.submit(); } return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>delete</B></FONT></A></TD>
+
+  <TD><A HREF=# onclick=\"var f=window.document.FileManager; if (get_num_checked() == 0) { window.alert('Please select a file to rename by clicking on a check box with the mouse.'); } else { f.FILEMANAGER_action.value='rename'; var rv=window.prompt('enter new name',''); if ((rv != null)&&(rv != '')) { f.FILEMANAGER_rename.value=rv; f.submit(); } } return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>rename</B></FONT></A></TD>
+
+  <TD><A HREF=# onclick=\"var f=window.document.FileManager; if (get_num_checked() == 0) { window.alert('Please select a file to extract by clicking on a check box with the mouse.'); } else { f.FILEMANAGER_action.value='extract'; f.submit(); } return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>extract</B></FONT></A></TD>
+
   <TD><A HREF=# onclick=\"var f=window.document.FileManager; f.FILEMANAGER_action.value='mkdir'; var rv=window.prompt('new directory name',''); if ((rv != null)&&(rv != '')) { f.FILEMANAGER_new_dir.value=rv; f.submit(); } return false;\" style='text-decoration:none'><FONT COLOR=WHITE><B>new directory</B></FONT></A></TD>
   <TD><A HREF=# style='text-decoration:none' onclick=\"window.print_upload(); return false;\"><FONT COLOR=WHITE><B>upload<B></FONT></A></TD>
 $rsync
 
-
 </TD></TR></TABLE></TD>
 </TR>
 
@@ -513,44 +607,22 @@ sub dir_link_ctl {
   my @loc = split /\//, r->param('FILEMANAGER_curr_dir');
 
   #already in base directory?
-  return "/" if ($#loc == -1);
+  return "<B>/</B>" if ($#loc == -1);
 
   #for all elements in the loc except the last one
   my @ac;
   for (my $i = 0; $i < $#loc; $i++) {
     push @ac, $loc[$i];
     my $url = join("/", @ac);
-    $loc[$i] = "<A HREF=# onclick=\"var f=window.document.FileManager; f.FILEMANAGER_curr_dir.value='$url'; f.submit(); return false;\" style='text-decoration:none'><FONT COLOR=#006699>".$loc[$i]."</FONT></A>";
+    $loc[$i] = "<A HREF=# onclick=\"var f=window.document.FileManager; f.FILEMANAGER_curr_dir.value='$url'; f.submit(); return false;\" style='text-decoration:none'><FONT COLOR=#006699 SIZE=+1><B>".$loc[$i]."</B></FONT></A>";
   }
 
+  $loc[$#loc] = "<B><FONT SIZE=+1>".$loc[$#loc]."</FONT></B>";
+
   return "<A HREF=# onclick=\"var f=window.document.FileManager; f.FILEMANAGER_curr_dir.value=''; f.submit(); return false;\" style='text-decoration:none'><FONT COLOR=#006699 SIZE=+1><B>/</B></FONT></A>&nbsp;".join("/&nbsp;", @loc);
 }
 
 
-#create a string representation of the perl structure
-sub stringify {
-  my $ref = shift;
-  my $rv;
-  eval {
-    $rv = encode_base64(freeze($ref),"");
-  }; if ($@) {
-    warn $@;
-    return undef;
-  }
-  return $rv;
-}
-
-#create the structure from the string representation
-sub destringify {
-  my $arg = shift || return undef;
-  my $rv;
-  eval {
-    $rv = thaw(decode_base64($arg));
-  };
-  warn $@ if $@;
-  return $rv;
-}
-
 #escape spaces in filename
 sub fn_esc {
   my $f = shift;
